import sys

import tuigpt.send as send



def main():
    chat = [[]]

    print('tui g4f')
    print('@by_chesnok')
    print('press exit to exit')
    print('=====')

    while 1:
        q = input('input: ')
        if q == 'exit':
            sys.exit()
        chat[0].append('user')
        chat[0].append(q)
        result = send.request(chat)
        chat[0].append('gpt')
        chat[0].append(result)
        print(f'gpt: {result}')
        
        # print(chat)







if __name__ == '__main__':
    try:
        main()
    except Exception as e:
        print(e)


from __future__ import annotations

import asyncio
from asyncio import SelectorEventLoop
from abc import ABC, abstractmethod

import browser_cookie3

from ..typing import AsyncGenerator, CreateResult


class BaseProvider(ABC):
    url: str
    working               = True
    needs_auth            = False
    supports_stream       = False
    supports_gpt_35_turbo = False
    supports_gpt_4        = False

    @staticmethod
    @abstractmethod
    def create_completion(
        model: str,
        messages: list[dict[str, str]],
        stream: bool,
        **kwargs
    ) -> CreateResult:
        raise NotImplementedError()


    @classmethod
    @property
    def params(cls):
        params = [
            ("model", "str"),
            ("messages", "list[dict[str, str]]"),
            ("stream", "bool"),
        ]
        param = ", ".join([": ".join(p) for p in params])
        return f"g4f.provider.{cls.__name__} supports: ({param})"


class AsyncProvider(BaseProvider):
    @classmethod
    def create_completion(
        cls,
        model: str,
        messages: list[dict[str, str]],
        stream: bool = False,
        **kwargs
    ) -> CreateResult:
        loop = create_event_loop()
        try:
            yield loop.run_until_complete(cls.create_async(model, messages, **kwargs))
        finally:
            loop.close()

    @staticmethod
    @abstractmethod
    async def create_async(
        model: str,
        messages: list[dict[str, str]],
        **kwargs
    ) -> str:
        raise NotImplementedError()


class AsyncGeneratorProvider(AsyncProvider):
    supports_stream = True

    @classmethod
    def create_completion(
        cls,
        model: str,
        messages: list[dict[str, str]],
        stream: bool = True,
        **kwargs
    ) -> CreateResult:
        loop = create_event_loop()
        try:
            generator = cls.create_async_generator(
                model,
                messages,
                stream=stream,
                **kwargs
            )
            gen  = generator.__aiter__()
            while True:
                try:
                    yield loop.run_until_complete(gen.__anext__())
                except StopAsyncIteration:
                    break
        finally:
            loop.close()

    @classmethod
    async def create_async(
        cls,
        model: str,
        messages: list[dict[str, str]],
        **kwargs
    ) -> str:
        return "".join([
            chunk async for chunk in cls.create_async_generator(
                model,
                messages,
                stream=False,
                **kwargs
            )
        ])
        
    @staticmethod
    @abstractmethod
    def create_async_generator(
        model: str,
        messages: list[dict[str, str]],
        **kwargs
    ) -> AsyncGenerator:
        raise NotImplementedError()


# Don't create a new event loop in a running async loop.
# Force use selector event loop on windows and linux use it anyway.
def create_event_loop() -> SelectorEventLoop:
    try:
        asyncio.get_running_loop()
    except RuntimeError:
        return SelectorEventLoop()
    raise RuntimeError(
        'Use "create_async" instead of "create" function in a async loop.')


_cookies = {}

def get_cookies(cookie_domain: str) -> dict:
    if cookie_domain not in _cookies:
        _cookies[cookie_domain] = {}
        try:
            for cookie in browser_cookie3.load(cookie_domain):
                _cookies[cookie_domain][cookie.name] = cookie.value
        except:
            pass
    return _cookies[cookie_domain]


def format_prompt(messages: list[dict[str, str]], add_special_tokens=False):
    if add_special_tokens or len(messages) > 1:
        formatted = "\n".join(
            ["%s: %s" % ((message["role"]).capitalize(), message["content"]) for message in messages]
        )
        return f"{formatted}\nAssistant:"
    else:
        return messages[0]


import codecs
import os

from setuptools import find_packages, setup



with open("README.md") as f:
    long_description = "\n" + f.read()

# with open("./requirements.txt") as f:
#     required = f.read().splitlines()

required = """
requests
pycryptodome
curl_cffi
aiohttp
certifi
browser_cookie3
websockets
js2py
flask
flask-cors
typing-extensions
PyExecJS"""


VERSION = '0.4'
DESCRIPTION = (
    "tui gpt"
)

# Setting up
setup(
    name="tuigpt",
    version=VERSION,
    author="chesnok",
    description=DESCRIPTION,
    author_email='ChesnokovP1107@yandex.ru',
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(),
    data_files=["tuigpt/main.py"],
    install_requires=required,
    entry_points={
        "console_scripts": ["tuigpt=main:main"],
    },
    url="https://github.com/chesnokpeter/tuigpt",  # Link to your GitHub repository
    project_urls={
        "Source Code": "https://github.com/chesnokpeter/tuigpt",  # GitHub link
    },
    keywords=[
        "python",
        "chatbot",
        "reverse-engineering",
        "openai",
        "chatbots",
        "gpt",
        "language-model",
        "gpt-3",
        "gpt3",
        "openai-api",
        "gpt-4",
        "gpt4",
        "chatgpt",
        "chatgpt-api",
        "openai-chatgpt",
        "chatgpt-free",
        "chatgpt-4",
        "chatgpt4",
        "chatgpt4-api",
        "free",
        "free-gpt",
        "gpt4free",
        "g4f",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: Microsoft :: Windows",
    ],
)



from tuigpt.back.g4f import ChatCompletion
# from tuigpt.back.g4f.Provider.base_provider import BaseProvider
from tuigpt.back.g4f.Provider.GptGo import GptGo

def request(messages:list[list], model:str ='gpt-3.5-turbo', provider=GptGo, stream:bool=False) -> str:
    response = ChatCompletion.create(model=model, stream=False, messages=messages, provider=provider)
    return response


